@@ -1,9 +1,10 @@
 """Provide file load and save functionality for PBN, RBN and lin formats."""
 
-__all__ = ['load_pbn', 'save_pbn', 'load_lin', 'load_rbn', 'save_rbn', 'BoardAsString']
+__all__ = ['load_pbn', 'save_pbn', 'load_lin', 'load_rbn', 'parse_pbn', 'parse_lin', 'save_rbn', 'BoardAsString']
 
 import os
 import datetime
+import re
 
 from .event import Event
 from .board import Board
@@ -37,13 +38,16 @@ PBN_CREATOR_TEXT = '% Creator: bridgeobjects'
 RBN_VERSION_TEXT = '% RBN version 3.2'
 RBN_CREATOR_TEXT = '% Created by bridgeobjects'
 
+DEFAULT_EVENT_NAME = 'BfG default event name'
+DEFAULT_EVENT_DATE = datetime.date(1980, 1, 1)
+
 
 def load_pbn(path):
     """Load a pbn file to a list of events."""
     events = []
     file_text = _get_file_text(path)
     if file_text:
-        events = _parse_pbn(file_text)
+        events = parse_pbn(file_text)
     return events
 
 
@@ -79,7 +83,7 @@ def load_lin(path):
     events = []
     file_text = _get_file_text(path)
     if file_text:
-        events = _parse_lin(file_text)
+        events = parse_lin(file_text)
     return events
 
 
@@ -89,6 +93,7 @@ def _get_file_text(path):
     try:
         with open(path, "r") as pbn_file:
             raw_text = pbn_file.read()
+            raw_text = raw_text.replace(chr(13), '')
             raw_text = raw_text.replace('\r', '')
             file_text = raw_text.split('\n')
     except FileNotFoundError:
@@ -96,15 +101,19 @@ def _get_file_text(path):
     return file_text
 
 
-def _parse_pbn(file_text):
+def parse_pbn(file_lines):
     """Populate bridgeobjects objects from a pbn file."""
-    events = []
     current_name = None
     current_board_identifier = ""
     auction_state = False
     play_state = False
     board_state = False
-    for line in file_text:
+    events = []
+    event = Event(DEFAULT_EVENT_NAME)
+    events.append(event)
+    board = None
+    value = None
+    for line in file_lines:
         if line:
             tag = ""
             if line.startswith("%"): # Comments
@@ -125,10 +134,9 @@ def _parse_pbn(file_text):
                 if current_name != value:
                     current_name = value
                     current_board_identifier = ""
-                    event = Event(value)
-                    events.append(event)
-                    auction_calls = []
-                    auction_note_keys = []
+                    if value and value != DEFAULT_EVENT_NAME:
+                        event = Event(value)
+                        events.append(event)
 
             # Event location
             elif tag == "Site":
@@ -206,7 +214,7 @@ def _parse_pbn(file_text):
 # TODO sort out modifiers
             # Contract
             elif tag == "Contract":
-                if value[-1] == 'X':
+                if value and value[-1] == 'X':
                     if value[-2] == 'X':
                         board.contract.modifier = 'R'
                         value = value[:-2]
@@ -250,9 +258,13 @@ def _parse_pbn_line(line):
 def _parse_pbn_date(value):
     """Return a datetime date object from a pbn date string."""
     # example date value: 2018.02.26
-    date = value.split('.')
-    date = list(map(int, date))
-    return datetime.date(date[0], date[1], date[2])
+    is_valid = re.search("[0-9]{4}\.[\0-9]{2}\.[0-9]{2}", value)
+    if is_valid:
+        date = value.split('.')
+        date = list(map(int, date))
+        return datetime.date(date[0], date[1], date[2])
+    else:
+        return DEFAULT_EVENT_DATE
 
 
 def _parse_pbn_deal(board, deal):
@@ -780,7 +792,7 @@ def _write_file(output, path, append):
     return success
 
 
-def _parse_lin(file_text):
+def parse_lin(file_text):
     """Populate bridgeobjects objects from a pbn file."""
     events = []
     event = Event()
@@ -805,22 +817,23 @@ def _parse_lin(file_text):
 def _parse_lin_board(line):
     """Return a board object from a lin line."""
     board = Board()
-    text = line.split('|')
-    board.identifier = text[7]
-    board.vulnerable = LIN_VULNERABILITY_CONVERSION[text[9]]
-    board.dealer = ['S', 'W', 'N', 'E'][int(text[3][0]) - 1]
-    board.hands = _parse_lin_hands(text[3][1:], board.dealer)
+    line_list = line.split('|')
+    board.identifier = line_list[7]
+    board.vulnerable = LIN_VULNERABILITY_CONVERSION[line_list[9]]
+    dealer_index = int(line_list[3][0]) - 1
+    board.dealer = ['S', 'W', 'N', 'E'][dealer_index]
+    board.hands = _parse_lin_hands(line_list[3][1:], board.dealer)
     return board
 
 
-def _parse_lin_hands(text, dealer):
+def _parse_lin_hands(line_list, dealer):
     """Return a list of hands from lin text."""
-    hands = [Hand(), Hand(), Hand(), Hand()]
+    hands = {}
     raw_hands = []
     dealer_index = SEATS.index(dealer)
     hand_index = (dealer_index - 2) % 4
     hand = Hand()
-    for character in text:
+    for character in line_list:
         if character in Suit.SHORT_NAMES:
             suit_name = character
         elif character == ',':
@@ -839,7 +852,7 @@ def _parse_lin_hands(text, dealer):
     hand_index %= 4
 
     raw_hands.append(Hand())
-    for card in Board.build_fourth_hand(raw_hands):
+    for card in Board.build_fourth_hand(hands):
         hand.cards.append(card)
     hands[hand_index] = hand
     return hands
@@ -880,4 +893,4 @@ class BoardAsString(object):
                 note = '[Note "{}:{}"]'.format(key, board.play_notes[key])
                 output.append(note)
             output.append('')
-        return output
\ No newline at end of file
+        return output
