@@ -788,8 +788,12 @@ class Hand(object):
     def _get_suit_points(self, suit):
         """Return the high card points for the suit."""
         suit_points = 0
+        if isinstance(suit, Suit):
+            suit_name = suit.name
+        else:
+            suit_name = suit
         for card in self._cards:
-            if card.suit.name == suit.name:
+            if card.suit.name == suit_name:
                 if card.value >= 10:
                     suit_points += card.value-9
         return suit_points
