@@ -1,13 +1,11 @@
 """A set of tests for the bridgeobjects Board class."""
 import pytest
-from mock import patch
 from ..source.board import Board
 from ..source.hand import Hand
 from ..source.call import Call
 from ..source.card import Card
 from ..source.auction import Auction
 from ..source.contract import Contract
-from ..source.denomination import Denomination
 
 
 def test_board_instantiate():
